@@ -44,22 +44,26 @@ TreeSupport::TreeSupport(const SliceDataStorage& storage)
 
 void TreeSupport::generateSupportAreas(SliceDataStorage& storage)
 {
-    const bool global_use_tree_support = Application::getInstance().current_slice->scene.current_mesh_group->settings.get<bool>("support_tree_enable");
+    const Settings& group_settings = Application::getInstance().current_slice->scene.current_mesh_group->settings;
+    const bool global_use_tree_support =
+        group_settings.get<bool>("support_enable")&&
+        group_settings.get<ESupportStructure>("support_structure") == ESupportStructure::TREE;
 
     if (!(global_use_tree_support || 
           std::any_of(storage.meshes.cbegin(),
                       storage.meshes.cend(),
                       [](const SliceMeshStorage& m) { 
-                          return m.settings.get<bool>("support_tree_enable");
+                          return m.settings.get<bool>("support_enable") &&
+                                 m.settings.get<ESupportStructure>("support_structure") == ESupportStructure::TREE;
                       })))
     {
         return;
     }
 
-    std::vector<std::unordered_set<Node*>> contact_nodes(storage.support.supportLayers.size()); //Generate empty layers to store the points in.
+    std::vector<std::vector<Node*>> contact_nodes(storage.support.supportLayers.size()); //Generate empty layers to store the points in.
     for (SliceMeshStorage& mesh : storage.meshes)
     {
-        if (mesh.settings.get<bool>("support_tree_enable"))
+        if (mesh.settings.get<ESupportStructure>("support_structure") == ESupportStructure::TREE)
         {
             generateContactPoints(mesh, contact_nodes);
         }
@@ -84,7 +88,7 @@ void TreeSupport::generateSupportAreas(SliceDataStorage& storage)
     storage.support.generated = true;
 }
 
-void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::unordered_set<Node*>>& contact_nodes)
+void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::vector<Node*>>& contact_nodes)
 {
     const Settings& mesh_group_settings = Application::getInstance().current_slice->scene.current_mesh_group->settings;
     const coord_t branch_radius = mesh_group_settings.get<coord_t>("support_tree_branch_diameter") / 2;
@@ -98,7 +102,7 @@ void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::
     const coord_t circle_side_length = 2 * branch_radius * sin(M_PI / CIRCLE_RESOLUTION); //Side length of a regular polygon.
     const coord_t z_distance_bottom = mesh_group_settings.get<coord_t>("support_bottom_distance");
     const coord_t layer_height = mesh_group_settings.get<coord_t>("layer_height");
-    const size_t z_distance_bottom_layers = round_up_divide(z_distance_bottom, layer_height);
+    const size_t z_distance_bottom_layers = round_up_divide(z_distance_bottom, layer_height) > 0 ? round_up_divide(z_distance_bottom, layer_height) : 1;
     const size_t tip_layers = branch_radius / layer_height; //The number of layers to be shrinking the circle to create a tip. This produces a 45 degree angle.
     const double diameter_angle_scale_factor = sin(mesh_group_settings.get<AngleRadians>("support_tree_branch_diameter_angle")) * layer_height / branch_radius; //Scale factor per layer to produce the desired angle.
     const coord_t line_width = mesh_group_settings.get<coord_t>("support_line_width");
@@ -132,7 +136,7 @@ void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::
                 }
                 circle.add(node.position + corner);
             }
-            if (node.support_roof_layers_below >= 0)
+            if (node.support_roof_layers_below > 0)
             {
                 roof_layer.add(circle);
             }
@@ -143,10 +147,10 @@ void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::
         }
         support_layer = support_layer.unionPolygons();
         roof_layer = roof_layer.unionPolygons();
-        support_layer = support_layer.difference(roof_layer);
         const size_t z_collision_layer = static_cast<size_t>(std::max(0, static_cast<int>(layer_nr) - static_cast<int>(z_distance_bottom_layers) + 1)); //Layer to test against to create a Z-distance.
         support_layer = support_layer.difference(volumes_.getCollision(0, z_collision_layer)); //Subtract the model itself (sample 0 is with 0 diameter but proper X/Y offset).
-        roof_layer = roof_layer.difference(volumes_.getCollision(0, z_collision_layer));
+        roof_layer = roof_layer.difference(volumes_.getCollision(branch_radius, z_collision_layer));
+        support_layer = support_layer.difference(roof_layer);
         //We smooth this support as much as possible without altering single circles. So we remove any line less than the side length of those circles.
         const double diameter_angle_scale_factor_this_layer = static_cast<double>(storage.support.supportLayers.size() - layer_nr - tip_layers) * diameter_angle_scale_factor; //Maximum scale factor.
         support_layer.simplify(circle_side_length * (1 + diameter_angle_scale_factor_this_layer), resolution); //Don't deviate more than the collision resolution so that the lines still stack properly.
@@ -176,12 +180,12 @@ void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::
             support_layer = support_layer.difference(floor_layer.offset(10)); //Subtract the support floor from the normal support.
         }
 
-        for (PolygonRef part : support_layer) //Convert every part into a PolygonsPart for the support.
+        std::vector<PolygonsPart> support_layer_parts = support_layer.splitIntoParts();
+        for (PolygonsPart& part : support_layer_parts) //Convert every part into a PolygonsPart for the support.
         {
-            PolygonsPart outline;
-            outline.add(part);
-            storage.support.supportLayers[layer_nr].support_infill_parts.emplace_back(outline, line_width, wall_count);
+            storage.support.supportLayers[layer_nr].support_infill_parts.emplace_back(part, line_width, wall_count);
         }
+
 #pragma omp critical (support_max_layer_nr)
         {
             if (!storage.support.supportLayers[layer_nr].support_infill_parts.empty() || !storage.support.supportLayers[layer_nr].support_roof.empty())
@@ -201,7 +205,7 @@ void TreeSupport::drawCircles(SliceDataStorage& storage, const std::vector<std::
     }
 }
 
-void TreeSupport::dropNodes(std::vector<std::unordered_set<Node*>>& contact_nodes)
+void TreeSupport::dropNodes(std::vector<std::vector<Node*>>& contact_nodes)
 {
     const Settings& mesh_group_settings = Application::getInstance().current_slice->scene.current_mesh_group->settings;
     //Use Minimum Spanning Tree to connect the points on each layer and move them while dropping them down.
@@ -273,15 +277,16 @@ void TreeSupport::dropNodes(std::vector<std::unordered_set<Node*>>& contact_node
             }
             //Put it in the best one.
             nodes_per_part[closest_part + 1][node.position] = p_node; //Index + 1 because the 0th index is the outside part.
+
         }
         //Create a MST for every part.
         std::vector<MinimumSpanningTree> spanning_trees;
         for (const std::unordered_map<Point, Node*>& group : nodes_per_part)
         {
-            std::unordered_set<Point> points_to_buildplate;
+            std::vector<Point> points_to_buildplate;
             for (const std::pair<Point, Node*>& entry : group)
             {
-                points_to_buildplate.insert(entry.first); //Just the position of the node.
+                points_to_buildplate.emplace_back(entry.first); //Just the position of the node.
             }
             spanning_trees.emplace_back(points_to_buildplate);
         }
@@ -464,11 +469,17 @@ void TreeSupport::dropNodes(std::vector<std::unordered_set<Node*>>& contact_node
             Node* i_node = entry.second;
             for (size_t i_layer = entry.first; i_node != nullptr; ++i_layer, i_node = i_node->parent)
             {
-                contact_nodes[i_layer].erase(i_node);
-                to_free_node_set.insert(i_node);
-                for (Node* neighbour : i_node->merged_neighbours)
+                std::vector<Node*>::iterator to_erase = std::find(contact_nodes[i_layer].begin(), contact_nodes[i_layer].end(), i_node);
+                if (to_erase != contact_nodes[i_layer].end())
                 {
-                    unsupported_branch_leaves.push_front({i_layer, neighbour});
+                    to_free_node_set.insert(*to_erase);
+                    contact_nodes[i_layer].erase(to_erase);
+                    to_free_node_set.insert(i_node);
+                    
+                    for (Node* neighbour : i_node->merged_neighbours)
+                    {
+                        unsupported_branch_leaves.push_front({ i_layer, neighbour });
+                    }
                 }
             }
         }
@@ -485,7 +496,7 @@ void TreeSupport::dropNodes(std::vector<std::unordered_set<Node*>>& contact_node
     to_free_node_set.clear();
 }
 
-void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vector<std::unordered_set<TreeSupport::Node*>>& contact_nodes)
+void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vector<std::vector<TreeSupport::Node*>>& contact_nodes)
 {
     const coord_t point_spread = mesh.settings.get<coord_t>("support_tree_branch_distance");
 
@@ -528,6 +539,7 @@ void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vecto
         }
     }
 
+
     const coord_t layer_height = mesh.settings.get<coord_t>("layer_height");
     const coord_t z_distance_top = mesh.settings.get<coord_t>("support_top_distance");
     const size_t z_distance_top_layers = round_up_divide(z_distance_top, layer_height) + 1; //Support must always be 1 layer below overhang.
@@ -568,7 +580,7 @@ void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vecto
                         constexpr size_t distance_to_top = 0;
                         constexpr bool to_buildplate = true;
                         Node* contact_node = new Node(candidate, distance_to_top, (layer_nr + z_distance_top_layers) % 2, support_roof_layers, to_buildplate, Node::NO_PARENT);
-                        contact_nodes[layer_nr].insert(contact_node);
+                        contact_nodes[layer_nr].emplace_back(contact_node);
                         added = true;
                     }
                 }
@@ -580,7 +592,7 @@ void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vecto
                 constexpr size_t distance_to_top = 0;
                 constexpr bool to_buildplate = true;
                 Node* contact_node = new Node(candidate, distance_to_top, layer_nr % 2, support_roof_layers, to_buildplate, Node::NO_PARENT);
-                contact_nodes[layer_nr].insert(contact_node);
+                contact_nodes[layer_nr].emplace_back(contact_node);
             }
         }
         for (const ConstPolygonRef overhang_part : overhang)
@@ -603,12 +615,12 @@ void TreeSupport::generateContactPoints(const SliceMeshStorage& mesh, std::vecto
     }
 }
 
-void TreeSupport::insertDroppedNode(std::unordered_set<Node*>& nodes_layer, Node* p_node)
+void TreeSupport::insertDroppedNode(std::vector<Node*>& nodes_layer, Node* p_node)
 {
-    std::unordered_set<Node*>::iterator conflicting_node_it = nodes_layer.find(p_node);
+    std::vector<Node*>::iterator conflicting_node_it = std::find(nodes_layer.begin(), nodes_layer.end(), p_node);
     if (conflicting_node_it == nodes_layer.end()) //No conflict.
     {
-        nodes_layer.insert(p_node);
+        nodes_layer.emplace_back(p_node);
         return;
     }
 
@@ -751,4 +763,4 @@ Polygons ModelVolumes::calculateMachineBorderCollision(Polygon machine_border)
     return machine_volume_border;
 }
 
-} //namespace cura
\ No newline at end of file
+} //namespace cura
