@@ -233,24 +233,27 @@ unsigned int PolygonUtils::moveInside(const Polygons& polygons, Point& from, int
             continue;
         Point p0 = poly[poly.size()-2];
         Point p1 = poly.back();
+        // because we compare with vSize2 here (no division by zero), we also need to compare by vSize2 inside the loop
+        // to avoid integer rounding edge cases
         bool projected_p_beyond_prev_segment = dot(p1 - p0, from - p0) >= vSize2(p1 - p0);
         for(const Point& p2 : poly)
         {   
-            // X = A + Normal( B - A ) * ((( B - A ) dot ( P - A )) / VSize( A - B ));
+            // X = A + Normal(B-A) * (((B-A) dot (P-A)) / VSize(B-A));
+            //   = A +       (B-A) *  ((B-A) dot (P-A)) / VSize2(B-A);
             // X = P projected on AB
             const Point& a = p1;
             const Point& b = p2;
             const Point& p = from;
             Point ab = b - a;
             Point ap = p - a;
-            int64_t ab_length = vSize(ab);
-            if(ab_length <= 0) //A = B, i.e. the input polygon had two adjacent points on top of each other.
+            int64_t ab_length2 = vSize2(ab);
+            if(ab_length2 <= 0) //A = B, i.e. the input polygon had two adjacent points on top of each other.
             {
                 p1 = p2; //Skip only one of the points.
                 continue;
             }
-            int64_t ax_length = dot(ab, ap) / ab_length;
-            if (ax_length <= 0) // x is projected to before ab
+            int64_t dot_prod = dot(ab, ap);
+            if (dot_prod <= 0) // x is projected to before ab
             {
                 if (projected_p_beyond_prev_segment)
                 { //  case which looks like:   > .
@@ -280,7 +283,7 @@ unsigned int PolygonUtils::moveInside(const Polygons& polygons, Point& from, int
                     continue;
                 }
             }
-            else if (ax_length >= ab_length) // x is projected to beyond ab
+            else if (dot_prod >= ab_length2) // x is projected to beyond ab
             {
                 projected_p_beyond_prev_segment = true;
                 p0 = p1;
@@ -290,7 +293,7 @@ unsigned int PolygonUtils::moveInside(const Polygons& polygons, Point& from, int
             else 
             { // x is projected to a point properly on the line segment (not onto a vertex). The case which looks like | .
                 projected_p_beyond_prev_segment = false;
-                Point x = a + ab * ax_length / ab_length;
+                Point x = a + ab * dot_prod / ab_length2;
 
                 int64_t dist2 = vSize2(p - x);
                 if (dist2 < bestDist2)
@@ -352,24 +355,27 @@ unsigned int PolygonUtils::moveInside(const ConstPolygonRef polygon, Point& from
     }
     Point p0 = polygon[polygon.size() - 2];
     Point p1 = polygon.back();
+    // because we compare with vSize2 here (no division by zero), we also need to compare by vSize2 inside the loop
+    // to avoid integer rounding edge cases
     bool projected_p_beyond_prev_segment = dot(p1 - p0, from - p0) >= vSize2(p1 - p0);
     for(const Point& p2 : polygon)
     {
-        // X = A + Normal( B - A ) * ((( B - A ) dot ( P - A )) / VSize( A - B ));
+        // X = A + Normal(B-A) * (((B-A) dot (P-A)) / VSize(B-A));
+        //   = A +       (B-A) *  ((B-A) dot (P-A)) / VSize2(B-A);
         // X = P projected on AB
         const Point& a = p1;
         const Point& b = p2;
         const Point& p = from;
         Point ab = b - a;
         Point ap = p - a;
-        int64_t ab_length = vSize(ab);
-        if(ab_length <= 0) //A = B, i.e. the input polygon had two adjacent points on top of each other.
+        int64_t ab_length2 = vSize2(ab);
+        if(ab_length2 <= 0) //A = B, i.e. the input polygon had two adjacent points on top of each other.
         {
             p1 = p2; //Skip only one of the points.
             continue;
         }
-        int64_t ax_length = dot(ab, ap) / ab_length;
-        if (ax_length <= 0) // x is projected to before ab
+        int64_t dot_prod = dot(ab, ap);
+        if (dot_prod <= 0) // x is projected to before ab
         {
             if (projected_p_beyond_prev_segment)
             { //  case which looks like:   > .
@@ -401,7 +407,7 @@ unsigned int PolygonUtils::moveInside(const ConstPolygonRef polygon, Point& from
                 continue;
             }
         }
-        else if (ax_length >= ab_length) // x is projected to beyond ab
+        else if (dot_prod >= ab_length2) // x is projected to beyond ab
         {
             projected_p_beyond_prev_segment = true;
             p0 = p1;
@@ -411,7 +417,7 @@ unsigned int PolygonUtils::moveInside(const ConstPolygonRef polygon, Point& from
         else
         { // x is projected to a point properly on the line segment (not onto a vertex). The case which looks like | .
             projected_p_beyond_prev_segment = false;
-            Point x = a + ab * ax_length / ab_length;
+            Point x = a + ab * dot_prod / ab_length2;
 
             int64_t dist2 = vSize2(p - x);
             if (dist2 < bestDist2)
